/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Digital I/O
 *********************************************************************************/

#include <avr\io.h>
#include <avr\interrupt.h>
#include <avr\sfr_defs.h>
#include <string.h>
#include <stdlib.h>

/************************************************************************ 
 * DEFINE-uri pentru parametrizarea modulului LCD
 ************************************************************************/
#define LcdDATA_DDR		DDRA			// Portul pe care conectam firele de date la LCD-ul
#define LcdDATA_PORT		PORTA
#define LcdDATA_PIN		PINA

#define LcdCMD_DDR		DDRA			// Portul pe care conectam firele de comenzi la LCD
#define LcdCMD_PORT		PORTA
#define LcdCMD_PIN		PINA

#define LcdD4			 PA0				// Pin-ul pentru firul de date D4 de pe LCD
#define LcdD5			 PA1				// Pin-ul pentru firul de date D5 de pe LCD
#define LcdD6			 PA2				// Pin-ul pentru firul de date D6 de pe LCD
#define LcdD7			 PA3				// Pin-ul pentru firul de date D7 de pe LCD

#define LcdRS			 PA4			    // Pinul pentru selectare operatie (LCD)
#define LcdRW			 PA5				// Pinul pentru Read/ Write (LCD)
#define LcdE			 PA6				// Pinul de Enable (LCD)

#define LCD_INSTR_4wire 		0x28 	// 4 fire date, font 5x8
#define LCD_INSTR_display 		0x0E 	// Display On, Cursor On, Blinking On ( 1 Display Cursor Blink )
#define LCD_INSTR_clearDisplay  	0x01 	// Clear Display
#define LCD_INSTR_returnHome		0x02 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_nextLine 		0xC0 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_gotoCGRAM		0x40	// go to Character Generator RAM
#define LCD_GO_BACK_ONE 		0x04		//sa se intoarca cursorul cu o pozitie
#define LCD_GO_FRONT_ONE 		0x06		//sa avanseze cursorul

/*************
 * API LCD   *
 *************/
void LCD_init();                // Initializare modul LCD.Trebuie apelata inainte de a se face orice operatie cu LCD-ul
void LCD_writeInstruction(unsigned char _instruction);	// Trimite o instructiune catre lcd (vezi datasheet)
void LCD_writeData(unsigned char _data);		// Trimite date catre LCD pentru afisare
void LCD_write(unsigned char _byte);			// trimite un bute catre LCD la modul general (nu conteaza daca e instructiune sau date)
void LCD_waitNotBusy();					// Asteptam pana cand lcd-ul devine disponibil pt o noua comanda
void LCD_waitInstructions(unsigned char _instructions);	// Asteapta un numar de cicli de ceas.
void LCD_print(char* _msg);				// Afiseaza imformatia pe LCD (doar 1 linie, primele 16 caractere)
void LCD_print2(char* _msg1, char* _msg2);		// Afisare pe 2 lini pe LCD
void LCD_printDecimal2u(unsigned int _n);		// Afisare numar in baza 10 pe LCD
void LCD_printHexa(unsigned int _n);			// Afisare numar in baza 16 pe LCD

int armat,no_sec,my_index;
char password[4],entered_password[4];

void initTimer()
{
	TIMSK |= (1 << OCIE1A);
	TCCR1A &= ~(1 << WGM11);
	TCCR1A &= ~(1 << WGM10);
	TCCR1B |= (1 << WGM12);
	TCCR1B &= ~(1 << WGM13);
	TCCR1B |= (1 << CS12);
	TCCR1B &= ~(1 << CS11);
	TCCR1B |= (1 << CS10);
	OCR1AH = 0x3D;
	OCR1AL = 0x09;
}

ISR(TIMER1_COMPA_vect)
{
	// schimbare stare LED
	int i;
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print("Remaining ");
	LCD_printDecimal2u(30-no_sec);
	LCD_print("sec");
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_print("Password: ");
	for (i=0;i<my_index;i++)
		LCD_print("*");
	no_sec++;	
	if (no_sec==30){
		//declansez buzzerul
		LCD_writeInstruction(LCD_INSTR_clearDisplay);
		LCD_print("Buzz");
		while(1);
	}
}

void read_password(int next_line,int fast){
	my_index=0;
	if (next_line)
		LCD_writeInstruction(LCD_INSTR_nextLine);
	while (1) //loop key check forever
	{		
		//first column
		
		PORTC =0b01111111;

		//check for rows
		// any function that serves pressed button

		if (!bit_is_set(PINC, 3)){
			while (!bit_is_set(PINC, 3));
			entered_password[my_index++]='1';
			if (fast) LCD_print("*");
		}

		if (!bit_is_set(PINC, 2)){
			while (!bit_is_set(PINC, 2));
			entered_password[my_index++]='4';
			if (fast) LCD_print("*");
		}

		if (!bit_is_set(PINC, 1)){
			while (!bit_is_set(PINC, 1));
			entered_password[my_index++]='7';
			if (fast) LCD_print("*");
		}

		//second column

		PORTC =0b10111111;

		if (!bit_is_set(PINC, 3)){
			while (!bit_is_set(PINC, 3));
			entered_password[my_index++]='2';
			if (fast) LCD_print("*");
		}

		if (!bit_is_set(PINC, 2)){
			while (!bit_is_set(PINC, 2));
			entered_password[my_index++]='5';
			if (fast) LCD_print("*");
		}

		if (!bit_is_set(PINC, 1)){
			while (!bit_is_set(PINC, 1));
			entered_password[my_index++]='8';
			if (fast) LCD_print("*");
		}

		if (!bit_is_set(PINC, 0)){
			while (!bit_is_set(PINC, 0));
			entered_password[my_index++]='0';
			if (fast) LCD_print("*");
		}

		//third column

		PORTC =0b11011111;

		if (!bit_is_set(PINC, 3)){
			while (!bit_is_set(PINC, 3));
			entered_password[my_index++]='3';
			if (fast) LCD_print("*");
		}

		if (!bit_is_set(PINC, 2)){
			while (!bit_is_set(PINC, 2));
			entered_password[my_index++]='6';
			if (fast) LCD_print("*");
		}

		if (!bit_is_set(PINC, 1)){
			while (!bit_is_set(PINC, 1));
			entered_password[my_index++]='9';
			if (fast) LCD_print("*");
		}
		
		//fourth column

		PORTC =0b11101111;

		//butonul de backspace
		if (!bit_is_set(PINC, 3)){
			while (!bit_is_set(PINC, 3));
			if (my_index>0){
				my_index--;	//sa sterg ultimul caracter
				if (fast){
					LCD_writeInstruction(LCD_GO_BACK_ONE);
					LCD_print(" ");
					LCD_print(" ");	//sterg steluta anterioara
					LCD_writeInstruction(LCD_GO_FRONT_ONE);	
					if (my_index==0)
						LCD_print(" ");
					else 
						LCD_print("*");
				}
				//setarea anterioara
			}
		}

		//butonul de enter
		if (!bit_is_set(PINC, 2)){
			while (!bit_is_set(PINC, 2));
			if (my_index==4)	//doar daca s-au introdus 4 caractere las sa treaca mai departe
				break;
		}		
	}
}

void new_password(){
	int i,the_same;

	do{
		the_same=1;
		LCD_writeInstruction(LCD_INSTR_clearDisplay);
		LCD_print("Old password:");
		//citire parola
		read_password(1,1);
		
		//comparare cu parola interna(password==enter_password)
		for (i=0;i<4;i++)
			if (password[i]!=entered_password[i])
				the_same=0;
	}while(!the_same);
	//daca parola veche este cea buna
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print("New password:");
	read_password(1,1);
	
	//copiez in password enter_password
	for (i=0;i<4;i++)
		password[i]=entered_password[i];	
}

void alarm_on(){
	int the_same,i;
	no_sec=0;
	my_index=0;
	PORTD|=_BV(PD7);
	sei();
	//introducerea parolei pentru armare
	//pornesc alarma	
	read_password(0,0);
	cli();
	the_same=1;
	for (i=0;i<4;i++)
			if (password[i]!=entered_password[i])
				the_same=0;
	//declansare alarma
	if (the_same==0){
		//declansez buzzerul
		LCD_writeInstruction(LCD_INSTR_clearDisplay);
		LCD_print("Buzz!!!");
		while(1);
	}
	//cat timp a mai ramas
	
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print("Alarm on!");
	//led-ul semnalizeaza
	
	//astept sa fiu declansata
	no_sec=1;
	while(1){
		if (!bit_is_set(PIND,PD6)){
			while(!bit_is_set(PIND,PD6));
			my_index=0;
			sei();
			read_password(0,0);
			cli();			
			the_same=1;
			//verificare daca parola introdusa este cea corecta
			for (i=0;i<4;i++)
					if (password[i]!=entered_password[i])
						the_same=0;
			//declansare alarma
			if (the_same==0){
				//declansez buzzerul
				LCD_writeInstruction(LCD_INSTR_clearDisplay);
				LCD_print("Buzz!!!");
				while(1);
			}
			//parola a fost introdusa corect
			PORTD&=~_BV(PD7);	//alarma off din nou
			break;
		}
	}
}

int ledPin = 13;                // choose the pin for the LED
int inputPin = 2;               // choose the input pin (for PIR sensor)
int pirState = LOW;             // we start, assuming no motion detected
int val = 0;                    // variable for reading the pin status
 
void setup() {
  pinMode(ledPin, OUTPUT);      // declare LED as output
  pinMode(inputPin, INPUT);     // declare sensor as input
 
  Serial.begin(9600);


void loop(){
  val = digitalRead(inputPin);  // read input value
  if (val == HIGH) {            // check if the input is HIGH
    digitalWrite(ledPin, HIGH);  // turn LED ON
    if (pirState == LOW) {
      // we have just turned on
      Serial.println("Motion detected!");
      // We only want to print on the output change, not state
      pirState = HIGH;
    }
  } else {
    digitalWrite(ledPin, LOW); // turn LED OFF
    if (pirState == HIGH){
      // we have just turned of
      Serial.println("Motion ended!");
      // We only want to print on the output change, not state
      pirState = LOW;
    }
  }
}
//consider intreruperea data de apasarea butonului doar daca alarma este armata

int main()
{
    LCD_init();
	initTimer();
	
	LCD_print ("Alarma off");
	//setare parola initiala
	password[0]='0';
	password[1]='7';
	password[2]='1';
	password[3]='0';

	//high nibble for output(columns) low for input(rows);

	DDRC=0xF0;

	//enable internal pullups for PC0-PC3

	PORTC=0x0F;

	//activez rezistenta pull up pentru buton legat la D6
	DDRD|=_BV(PD7);
	PORTD|=_BV(PD6);
	PORTD&=~_BV(PD7);
			
	//astept fie ca alarma sa fie armata, fie sa se doreasca schimbarea parolei
	
	while (1) //loop key check forever
	{
		//first column
		PORTC =0b01111111;
		
		//s-a apasat *
		if (!bit_is_set(PINC, 0)){
			while (!bit_is_set(PINC, 0));
			new_password();
			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print("Alarm off");
		}

		//third column

		PORTC =0b11011111;

		//s-a apasat #
		if (!bit_is_set(PINC, 0)){
			while (!bit_is_set(PINC, 0));
			alarm_on();
			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print("Alarm off");
		}
		
	}
		
	return 0;
}

/*****************************
 * IMPLEMENTARE API LCD.     *
 *****************************/

void LCD_init()
{
	LcdDATA_DDR |=  (1<<LcdD4)|(1<<LcdD5)|(1<<LcdD6)|(1<<LcdD7);	// setam pinii de date ca pini de iesire
	LcdCMD_DDR  |=  (1<<LcdRS)|(1<<LcdRW)|(1<<LcdE);				// setam pinii de comenzi ca pini de iesire

	LCD_waitNotBusy();
	
	LcdCMD_PORT   &= ~(1<<LcdRS);									// Setam linia RS pe low
	LcdCMD_PORT   &= ~(1<<LcdRW);									// Setam linia RW pe low (acum suntem in modul de trimis instructiuni)
	LcdDATA_PORT  &= ~(1<<LcdD4)&~(1<<LcdD6)&~(1<<LcdD7); 		// Specificam ca vrem 4 fire de date, prima comanda (LcdD5 activ, restul nu)
	LcdDATA_PORT  |=  (1<<LcdD5);									// setam pinii de comenzi ca pini de iesire
	
	LcdCMD_PORT |=  (1<<LcdE);					// Setam linia E(nable) pe high; aceasta ii specifica LCD-ului sa preia datele
	LCD_waitInstructions(6);					// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE );				// Setam linia E(nable) pe low; transferul s-a terminat

	LCD_writeInstruction(LCD_INSTR_4wire); 		// Incarcam comanda: 4 bit data, 2 lines, 5x8 font
	LCD_writeInstruction(LCD_INSTR_display); 	// Display On, Cursor On, Blinking On
	LCD_writeInstruction(0x06);					// Increment, no shift
	LCD_writeInstruction(0x01);					// Clear Display

	
}

void LCD_writeInstruction(unsigned char _instruction)
{					
	LCD_waitNotBusy();// asteptam ca LCD-ul sa fie liber sa primeasca comenzile	
	LcdCMD_PORT &= ~(1 << LcdRS);// setam pinul RS pe low (low=instructiuni, high=date)
	LcdCMD_PORT &= ~(1<< LcdRW);// setam pinul RW pe low (suntem in modul de comenzi acum)
	LCD_write (_instruction);// apelam procedura ce trimite byte-ul pe firele de date
}


void LCD_writeData(unsigned char _data)
{
	// similar cu LCD_writeInstruction
	LCD_waitNotBusy();// asteptam ca LCD-ul sa fie liber sa primeasca comenzile	
	LcdCMD_PORT |= (1 << LcdRS);
	LcdCMD_PORT &= ~(1<< LcdRW);
	LCD_write (_data);// apelam procedura ce trimite byte-ul pe firele de date

}

void LCD_write(unsigned char _byte)
{
	// scriem cei mai semnificativi 4 biti pe firele de date
	if (bit_is_set(_byte, 7))
		LcdCMD_PORT |= (1<<LcdD7);
	else
		LcdCMD_PORT &= ~ (1 << LcdD7);
	if (bit_is_set(_byte, 6))
		LcdCMD_PORT |= (1<<LcdD6);
	else
		LcdCMD_PORT &= ~ (1 << LcdD6);
	if (bit_is_set(_byte, 5))
		LcdCMD_PORT |= (1<<LcdD5);
	else
		LcdCMD_PORT &= ~ (1 << LcdD5);
	if (bit_is_set(_byte, 4))
		LcdCMD_PORT |= (1<<LcdD4);
	else
		LcdCMD_PORT &= ~ (1 << LcdD4);
	// Setam Pinul E pe high
	LcdCMD_PORT |= (1 << LcdE);
	// Asteptam o perioada de timp T
	LCD_waitInstructions (10);
	// Setam Pinul E pe low
	LcdCMD_PORT &= ~(1 << LcdE);
	// Asteptam o perioada de timp T
	LCD_waitInstructions (10);
	// scriem cei mai putin semnificativi 4 biti pe firele de date
	if (bit_is_set(_byte, 3))
		LcdCMD_PORT |= (1<<LcdD7);
	else
		LcdCMD_PORT &= ~ (1 << LcdD7);
	if (bit_is_set(_byte, 2))
		LcdCMD_PORT |= (1<<LcdD6);
	else
		LcdCMD_PORT &= ~ (1 << LcdD6);
	if (bit_is_set(_byte, 1))
		LcdCMD_PORT |= (1<<LcdD5);
	else
		LcdCMD_PORT &= ~ (1 << LcdD5);
	if (bit_is_set(_byte, 0))
		LcdCMD_PORT |= (1<<LcdD4);
	else
		LcdCMD_PORT &= ~ (1 << LcdD4);
	// Setam Pinul E pe high
	LcdCMD_PORT |= (1 << LcdE);
	// Asteptam o perioada de timp T
	LCD_waitInstructions (10);
	// Setam Pinul E pe low
	LcdCMD_PORT &= ~(1 << LcdE);
}

void LCD_waitNotBusy()
{
	unsigned char _loop = 1;

	while (_loop)
	{
		LcdDATA_DDR &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7);	// Setam pinii de date de la LCD pe in pt a citi busy flag
		LcdDATA_PORT &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); 	// Dezactivam pullup resistor pentru pinii de in
		
		LcdCMD_PORT &= ~(1<<LcdE);					// Setam pin-ul e pe low; ar trebui sa fie deja low, doar ne asiguram
		LcdCMD_PORT &= ~(1<<LcdRS);					// Setam pinul RS pe low
		LcdCMD_PORT |=  (1<<LcdRW);					// Setam pinul RW pe high (acum suntem in modul de interogare busy/adr)
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		_loop = LcdDATA_PIN & (1<<LcdD7);				// Citim busy flag-ul
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
		
		LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

		LcdDATA_DDR |= (1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); // Setam Portul de LCD ca port de iesire la loc
	}
}

void LCD_waitInstructions(unsigned char _instructions)
{
	while (_instructions--)
		;
}


void LCD_printDecimal2u(unsigned int _n)
{
	unsigned char tmp=0;
	
	// Extragem sutele
	while(_n>=100)
		_n-=100;

	while(_n>=10){
		tmp++;
		_n-=10;
	}

	LCD_writeData(tmp+'0');
	LCD_writeData(_n+'0');
}

void LCD_printHexa(unsigned int _n)
{
	unsigned char _tmp = _n>>4;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
	_tmp = _n & 0x0F;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
}


void LCD_print(char* _msg)
{
	unsigned char i=0;
	for( ; _msg[i]!=0 && i<16; i++)
		LCD_writeData( _msg[i] );
}

void LCD_print2(char* _msg1, char* _msg2)
{
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print(_msg1);
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_print(_msg2);
}
